#ifndef H_TYPETREE
#define H_TYPETREE

#include <cstdint>
#include <vector>

class TypeTree {
public:
    struct Node {
        uint16_t type;
        size_t   parent;
        std::vector<size_t> children;
        
        Node(uint16_t type, size_t parent);
    };

    TypeTree();
    ~TypeTree();

    size_t get(size_t id, uint16_t type);
    uint16_t type(size_t id);
    size_t parent(size_t id);

private:
    std::vector<Node> m_nodes;
};

TypeTree::Node::Node(uint16_t type, size_t parent) :type(type), parent(parent) { }

TypeTree::TypeTree()
{
    m_nodes.push_back(Node(0, 0));
}

TypeTree::~TypeTree()
{ }

size_t TypeTree::get(size_t id, uint16_t type)
{
    size_t find = 0;

    if(id < m_nodes.size())
    {
        Node& node = m_nodes[id];

        for(size_t child : node.children) {
            if(m_nodes[child].type == type) {
                find = child;
                break;
            }
        }

        if(find == 0) {
            m_nodes.push_back(Node(type, id));
            find = m_nodes.size() - 1;

            node.children.push_back(find);
        }
    }

    return find;
}

uint16_t TypeTree::type(size_t id)
{
    if(id > 0 && id < m_nodes.size()) {
        return m_nodes[id].type;
    }
    return 0;
}

size_t TypeTree::parent(size_t id)
{
    if(id > 0 && id < m_nodes.size()) {
        return m_nodes[id].parent;
    }
    return 0;
}

#endif
