struct Node {
    pub class:u32,
    pub parent:usize,
    pub children:stdu::Sparse<usize>,
}
impl Node {
    pub fn new(class:u32, parent:usize) -> Self
    {
        Self {
            class:class,
            parent:parent,
            children:stdu::Sparse::<usize>::new(),
        }
    }
}

pub struct TypeTree {
    data:stdu::Pool<Node>,
}
impl TypeTree {
    pub fn new() -> Self
    {
        Self {
            data:stdu::Pool::<Node>::new(),
        }
    }

    pub fn add(&mut self, hierarchy:&Vec<u32>) -> usize
    // add
    //
    {
        let mut node = 0;
        for class in hierarchy {
            match self.data.get(node) {
                Some(parent) => match parent.children.get(*class as usize) {
                    Some(child) => {
                        node = *child;
                    }
                    None => {
                        let new_node = self.data.add(Node::new(*class, node));
                        self.data.get_mut(node).unwrap().children.set(*class as usize, new_node);
                        node = new_node;
                    }
                }
                None => { return 0; }
            }
        }
        return node;
    }

    pub fn insert(&mut self, id:usize, class:u32) -> Result<usize,()>
    // wrap an existing type
    //
    {
        let new_node = self.data.add(Node::new(class, id));
        return match self.data.get_mut(id) {
            Some(node) => {
                node.children.set(class as usize, new_node);
                Ok(new_node)
            }
            None => Err(()),
        }
    }

    pub fn inner(&self, id:usize) -> Option<usize>
    // returns an inner type if one exists
    //
    {
        return match self.data.get(id) {
            Some(node) => {
                if node.parent == 0 { None }
                else { Some(node.parent) }
            }
            None => None,
        }
    }

    fn lookup_node(&self, hierarchy:&Vec<u32>) -> (usize, usize)
    // finds the last existing node in the hierarchy
    {
        let mut index :usize = 0;
        let mut current :usize = 0;
        for i in 0..hierarchy.len() {
            match self.data.get(current).unwrap().children.get(hierarchy[i] as usize) {
                Some(child) => {
                    current = *child;
                    index += 1;
                }
                None => {
                    return (index, current);
                }
            }
        }
        return (0, 0);
    }
}
